package cn.newfeifan.mall.module.pay.service.fuyouorder;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.newfeifan.mall.framework.pay.core.client.PayClient;
import cn.newfeifan.mall.module.pay.dal.dataobject.channel.PayChannelDO;
import cn.newfeifan.mall.module.pay.dal.dataobject.order.PayOrderDO;
import cn.newfeifan.mall.module.pay.dal.dataobject.order.PayOrderExtensionDO;
import cn.newfeifan.mall.module.pay.dal.mysql.order.PayOrderExtensionMapper;
import cn.newfeifan.mall.module.pay.fuiou.reqvo.SubmitOrderRequestVO;
import cn.newfeifan.mall.module.pay.fuiou.respVO.FuYouPayOrderSubmitRespVO;
import cn.newfeifan.mall.module.pay.fuiou.respVO.FuYouPaymentResponseVO;
import cn.newfeifan.mall.module.pay.service.app.PayAppService;
import cn.newfeifan.mall.module.pay.service.channel.PayChannelService;
import cn.newfeifan.mall.module.pay.service.order.PayOrderService;
import com.fasterxml.jackson.databind.ObjectMapper;


import cn.newfeifan.mall.module.pay.fuiou.util.DateUtils;
import cn.newfeifan.mall.module.pay.fuiou.util.FuiouHttpPoster;
import cn.newfeifan.mall.module.pay.fuiou.util.MD5;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;

import java.time.Duration;
import java.time.LocalDateTime;

import static cn.newfeifan.mall.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.newfeifan.mall.framework.common.util.servlet.ServletUtils.getClientIP;
import static cn.newfeifan.mall.module.pay.enums.DictTypeConstants.*;
import static cn.newfeifan.mall.module.pay.enums.ErrorCodeConstants.CHANNEL_NOT_FOUND;
import static cn.newfeifan.mall.module.pay.enums.ErrorCodeConstants.SUBMIT_ORDER_ERROR;

@Service
@Validated
@Slf4j
public class FiYouPayOrderServiceImpl implements FuYouPayOrderService {

    @Value("${wx.mp.app-id}")
    private String appid;

    @Resource
    private PayOrderService payOrderService;
    @Resource
    StringRedisTemplate stringRedisTemplate;

    @Resource
    private PayOrderExtensionMapper orderExtensionMapper;

    @Resource
    private PayAppService appService;

    @Resource
    private PayChannelService channelService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String submitOrder(SubmitOrderRequestVO requestVO) {

        // 1.1 获得 PayOrderDO ，并校验其是否存在
        PayOrderDO payOrder = payOrderService.validateOrderCanSubmit(requestVO.getId());
        payOrder.setPayOrderNo(generate());
        payOrderService.updatePayOrder(payOrder);

        // 1.32 校验支付渠道是否有效
        PayChannelDO channel = validateChannelCanSubmit(payOrder.getAppId(), requestVO.getChannelCode());

        // 2. 插入 PayOrderExtensionDO
        PayOrderExtensionDO orderExtension = PayOrderExtensionDO.builder()
                .userIp(getClientIP())
                .orderId(payOrder.getId())
                .channelId(channel.getId())
                .channelCode(channel.getCode())
                .build();
        orderExtensionMapper.insert(orderExtension);

        FuiouHttpPoster http = new FuiouHttpPoster();
        http.setCharset("utf-8");
        http.setUrl("https://aipay.fuioupay.com/aggregatePay/wxPreCreate");
        requestVO.getReq().setTerm_id("88888888");
        requestVO.getReq().setTerm_ip("192.168.8.8");
        requestVO.getReq().setRandom_str(DateUtils.getCurrentDate("yyyyMMddHHmmss")
                + "568974");
        requestVO.getReq().setMchnt_cd(mchnt_cd);
        requestVO.getReq().setOrder_amt(String.valueOf(payOrder.getPrice()));
        requestVO.getReq().setGoods_des(payOrder.getBody());
        requestVO.getReq().setSub_appid(appid);
        requestVO.getReq().setMchnt_order_no(payOrder.getPayOrderNo());//不能重复
        requestVO.getReq().setTxn_begin_ts(DateUtils.getCurrentDate("yyyyMMddHHmmss"));
        requestVO.getReq().setNotify_url("https://letcgo.com/fuYou/pay/order/notify");
        requestVO.getReq().setVersion("1.0");
        StringBuilder sb = new StringBuilder();
        sb.append(requestVO.getReq().getMchnt_cd().trim()).append("|")
                .append(requestVO.getReq().getTrade_type().trim()).append("|")
                .append(requestVO.getReq().getOrder_amt().trim()).append("|")
                .append(requestVO.getReq().getMchnt_order_no().trim()).append("|")
                .append(requestVO.getReq().getTxn_begin_ts()).append("|")
                .append(requestVO.getReq().getGoods_des()).append("|")
                .append(requestVO.getReq().getTerm_id()).append("|")
                .append(requestVO.getReq().getTerm_ip()).append("|")
                .append(requestVO.getReq().getNotify_url()).append("|")
                .append(requestVO.getReq().getRandom_str()).append("|")
                .append(requestVO.getReq().getVersion()).append("|")
                .append(privateKey);
        requestVO.getReq().setSign(MD5.MD5Encode(sb.toString()));
        String res = null;
            log.info("向富友发起微信支付请求 == 请求体：{}", JSON.toJSONString(requestVO.getReq()));
        try {
            res = http.newPost(JSON.toJSONString(requestVO.getReq()));

            log.info("向富友发起微信支付请求 == 响应体：{}", res);
            // 解析返回结果,获取参数
            ObjectMapper mapper = new ObjectMapper();
            FuYouPaymentResponseVO resp = mapper.readValue(res, FuYouPaymentResponseVO.class);

            if (!"000000".equals(resp.getResult_code())) {
                throw exception(SUBMIT_ORDER_ERROR);
            }

            FuYouPayOrderSubmitRespVO vo = FuYouPayOrderSubmitRespVO.builder()
                    .appId(resp.getSdk_appid())
                    .timeStamp(resp.getSdk_timestamp())
                    .nonceStr(resp.getSdk_noncestr())
                    .sdk_package(resp.getSdk_package())
                    .signType(resp.getSdk_signtype())
                    .paySign(resp.getSdk_paysign())
                    .build();
            return JSON.toJSONString(vo);
        } catch (Exception e) {
            throw exception(SUBMIT_ORDER_ERROR);
        }
    }

    private PayChannelDO validateChannelCanSubmit(Long appId, String channelCode) {
        // 校验 App
        appService.validPayApp(appId);
        // 校验支付渠道是否有效
        PayChannelDO channel = channelService.validPayChannel(appId, channelCode);
        PayClient client = channelService.getPayClient(channel.getId());
        if (client == null) {
            log.error("[validatePayChannelCanSubmit][渠道编号({}) 找不到对应的支付客户端]", channel.getId());
            throw exception(CHANNEL_NOT_FOUND);
        }
        return channel;
    }

    private String generate() {
        // 递增序号
        String noPrefix = cn.newfeifan.mall.module.pay.enums.DictTypeConstants.FUYOU_TRADE_ORDER_NO_PREFIX + DateUtil.format(LocalDateTime.now(), DatePattern.PURE_DATETIME_PATTERN);
        String key = TRADE_NO + noPrefix;
        Long no = stringRedisTemplate.opsForValue().increment(key);
        // 设置过期时间
        stringRedisTemplate.expire(key, Duration.ofMinutes(1L));
        return noPrefix + no;
    }

    @Override
    public void afterSale(Long afterSaleId) {

    }

    /**
     * 生成随机字符串
     */
    private static String generateRandomString(int length) {
        String characters = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < length; i++) {
            int index = (int) (characters.length() * Math.random());
            sb.append(characters.charAt(index));
        }
        return sb.toString();
    }
}
